/********************************************************
 The %skmdata macro

 A SAS macro for the Kaplan-Meier plot with SGPLOT
 Procedure


 Copyright (c) 2015 Kengo NAGASHIMA

 This software is released under the MIT License,
 see <http://opensource.org/licenses/mit-license.php>.
********************************************************/

/********************************************************
 Author: Kengo NAGASHIMA
 Version: 1.0.0
 Last Updated Date: 03/06/2015
 System Requirements: SAS 9.4 or later, base SAS,
                      SAS/STAT 13.1 or later
********************************************************/

/********************************************************
 Arguments
   data: input dataset name
   time: time variable
   event: event variable
   group: group variable
   censor_value: code value of censored observations
   out: output dataset name
   timemax: maximum time for no. at risk order
   timeby: increment of the sequence for no. at risk order
********************************************************/

%macro skmdata(data, time, event, group, censor_value, out, timemax, timeby);
%if "&group." = "" %then %do;
data _tmp_; set &data.; _group_ = 1;
%let group = _group_;
%end;
%else %do;
data _tmp_; set &data.;
%end;

ods graphics off;
proc lifetest data = _tmp_ outsurv = _sdata_;
  time &time.*&event.(&censor_value.);
  strata &group.;
  ods output ProductLimitEstimates = _ple_;
run;
ods graphics on;
data _sdata_;
  set _sdata_;
  if _censor_ = 1 then ycensor = SURVIVAL;
  else ycensor = .;
proc sort data = _ple_; by STRATUM &time.;
data _ple_;
  set _ple_; by STRATUM &time.;
  do _x_ = 0 to &timemax. by &timeby.;
    if &time. = 0 & Censor = . then do; output; leave; end;
    else if &time. <= _x_ then do; output; leave; end;
  end;
data _ple_;
  format atrisk $5.;
  length atrisk $5.;
  set _ple_; by STRATUM _x_;
  if last._x_ then output;
  keep _x_ left &group.;
  rename _x_ = xatrisk left = atrisk;
data &out.;
  set _sdata_ _ple_;
proc datasets lib = work;
  delete _tmp_ _sdata_ _ple_;
run; quit;
%mend;
